#!/usr/bin/python
# vim: set fileencoding=utf-8 :

import os
import time
import sys
import psutil
import json
import logging
import logging.config

config = json.load(open(sys.argv[1]))
logging.config.dictConfig(config.get("logging", {}))

logger = logging.getLogger()


def restart(server):
    #  if 'port' in server:
    #      fierro_monitor = "---------------------------MONITOR DE FIERRO---------------------------"
    #      monitor = os.popen("links -dump http://fierro:123456@localhost:%s/basic" % (server['port'] + 200))
    #      for line in monitor.readlines():
    #          if "Sesiones:" in line:
    #              break
    #          logger.warning(line)
    # ESTA BUENA LA IDEA PERO HAY QUE USAR SUBPROCESS, VERIFICAR EL RETCODE porque puede
    # ser que no tenga monitor.
    # Y mandar al logger como un solo mensaje de muchas líneas, no varios.
    ret_code = os.system(server["restart_action"])
    if ret_code != 0:
        logger.error("Error restarting %s - ret_code: %s" % (get_name(server), ret_code))


def get_name(server):
    if "name" in server:
        return server["name"]
    else:
        return server["pid_file"]


def get_memory_info(process):
    "Adaptador por diferencia entre versiones de psutil"
    if hasattr(process, "get_memory_info"):
        return process.get_memory_info()
    else:
        return process.memory_info()


def check_fierro_response(server):
    try:
        import Pyro.core
    except ImportError:
        logger.warning("Error importing Pyro.core - unable to check fierro response")
        return True
    if "port" not in server:
        logger.warning("Server %s - Port not specified - unable to check fierro response", get_name(server))
        return True
    server_URI = "PYROLOC://localhost:%d/dispatcher" % server["port"]
    pyro_server = Pyro.core.getProxyForURI(server_URI)
    t0 = time.time()
    try:
        resp = pyro_server.run("/login", "getSucursales", ())
    except:
        logger.exception("Server %s not responding - Error calling getSucursales", get_name(server))
        return False
    if not resp:
        logger.error("Server %s - Empty response calling getSucursales", get_name(server))
        return False
    t1 = time.time()
    if (t1 - t0) > 10.0:
        logger.error("Server %s too slow - takes %.3f secs for getSucursales", get_name(server), t1 - t0)
        return False
    return True


logger.info("Controlling processes: %s" % [get_name(s) for s in config["servers"]])

for server in config["servers"]:
    logger.debug("Checking server %s" % server)
    try:
        pid_file = open(server["pid_file"])
    except:
        logger.error("Server %s - Error opening pidfile", get_name(server))
        pid = None
    else:
        pid = pid_file.read().strip()
    if not pid:
        if "port" in server:
            logger.warning("Server %s not running - empty pid - Restarting" % get_name(server))
            os.system("fuser -n tcp -k %s" % server["port"])
            time.sleep(2)
            restart(server)
        else:
            logger.warning("Server %s not running - empty pid - Port not specified" % get_name(server))
            restart(server)
        continue
    else:
        pid = int(pid)
    try:
        process = psutil.Process(pid)
    except psutil.NoSuchProcess:
        logger.warning("Server %s not running - Restarting" % get_name(server))
        restart(server)
        continue
    if get_memory_info(process).vms > server["max_memory"]:
        logger.warning("Server %s memory above threeshold: %s > %s - Restarting" %
                     (get_name(server), get_memory_info(process).vms, server["max_memory"]))
        restart(server)
        continue
    if server.get("check_fierro_response", False):
        if not check_fierro_response(server):
            restart(server)
